/*
 *  m_killfizzer.c: Crude hack to automatically KLINE or DLINE possible fizzer
 *                  clients.
 *
 *  Copyright 2003 by W. Campbell
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are
 *  met:
 *
 *  1.Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer. 
 *  2.Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution. 
 *  3.The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 *  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 *  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 *  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *
 *  $Id$
 *
 */

/* List of ircd includes from ../include/ */
#include "stdinc.h"
#include "handlers.h"
#include "client.h"
#include "common.h"     /* FALSE bleah */
#include "ircd.h"
#include "irc_string.h"
#include "numeric.h"
#include "fdlist.h"
#include "s_bsd.h"
#include "s_conf.h"
#include "s_log.h"
#include "s_serv.h"
#include "send.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"
#include "s_misc.h"

static void mo_killfizzer(struct Client *client_p, struct Client *source_p,
                     int parc, char *parv[]);

struct Message killfizzer_msgtab = {
  "KILLFIZZER", 0, 0, 1, 0, MFLG_SLOW, 0,
  {m_unregistered, m_not_oper, m_ignore, mo_killfizzer}
};

#ifndef STATIC_MODULES
void
_modinit(void)
{
  mod_add_cmd(&killfizzer_msgtab);
}

void
_moddeinit(void)
{
  mod_del_cmd(&killfizzer_msgtab);
}

const char *_version = "$Revision$";
#endif

#define KF_DLINE	1
#define KF_KLINE	2
#define KF_TEMPK	3

/*
** mo_killfizzer
** Syntax:  KILLFIZZER <D|K|T>
*/
static void mo_killfizzer(struct Client *client_p, struct Client *source_p,
                     int parc, char *parv[])
{		 
  int count = 0;
  struct Client *target_p;
  dlink_node *dl;
  char gecos_buf[REALLEN + 1];
  char *secondp;
  char *checkp;
  char newuser[USERLEN + 1];
  unsigned int type;
  const char *current_date;
  struct ConfItem *aconf;

  if (parc < 2)
  {
    sendto_one(source_p, ":%s NOTICE %s :Syntax:  /KILLFIZZER <D|K|T>",
               me.name, parv[0]);
    sendto_one(source_p, ":%s NOTICE %s :D = DLINE, K = KLINE (PERM), T = "
               "KLINE (TEMP 1440)", me.name, parv[0]);
    return;
  }

  switch(parv[1][0])
  {
    case 'D':
    case 'd':
        type = KF_DLINE;
        break;
    case 'K':
    case 'k':
        type = KF_KLINE;
        break;
    case 'T':
    case 't':
        type = KF_TEMPK;
        break;
    default:
        sendto_one(source_p, ":%s NOTICE %s :Syntax:  /KILLFIZZER <D|K|T>",
                   me.name, parv[0]);
        sendto_one(source_p, ":%s NOTICE %s :D = DLINE, K = KLINE (PERM), T = "
                   "KLINE (TEMP 1440)", me.name, parv[0]);
        return;
  }

  /* Local clients only */
  DLINK_FOREACH(dl, lclient_list.head)
  {
    target_p = dl->data;

    strlcpy(gecos_buf, target_p->info, REALLEN + 1);
    secondp = strchr(gecos_buf, ' ');
    if (secondp == NULL)
      continue; /* No known one word gecos's with Fizzers */

    *secondp = '\0';
    secondp++;
    checkp = strchr(secondp, ' ');
    if (checkp != NULL)
      continue; /* No known 3 or more word gecos's with Fizzers */

    snprintf(newuser, USERLEN + 1, "~%s%s", secondp, gecos_buf);

    if (strcmp(newuser, target_p->username) == 0) /* They're case sensitive */
    {
      count++;
      if (type == KF_DLINE)
      {
        aconf = make_conf();
        current_date = smalldate(CurrentTime);
        aconf->status = CONF_DLINE;
        DupString(aconf->host, target_p->localClient->sockhost);
        DupString(aconf->passwd, "Fizzer Client (Auto-DLINE)");
        add_conf_by_address(aconf->host, CONF_DLINE, NULL, aconf);
#ifdef CONFIG_H_LEVEL_7
        WriteKlineOrDline(
#else
        write_confitem(
#endif
                       DLINE_TYPE, source_p, NULL, aconf->host,
                       aconf->passwd, "", current_date, 0);
      }
      else if (type == KF_KLINE)
      {
        aconf = make_conf();
        aconf->status = CONF_KILL;
        DupString(aconf->user, "*");
        DupString(aconf->host, target_p->host);
        current_date = smalldate(CurrentTime);
        DupString(aconf->passwd, "Fizzer Client (Auto-KLINE)");
        add_conf_by_address(aconf->host, CONF_KILL, aconf->user, aconf);
#ifdef CONFIG_H_LEVEL_7
        WriteKlineOrDline(
#else
        write_confitem(
#endif
                       KLINE_TYPE, source_p, aconf->user, aconf->host,
                       aconf->passwd, "", current_date, 0);
      }
      else if (type == KF_TEMPK)
      {
        aconf = make_conf();
        aconf->status = CONF_KILL;
        DupString(aconf->user, "*");
        DupString(aconf->host, target_p->host);
        DupString(aconf->passwd, "Fizzer Client (Auto-KLINE)");
        aconf->hold = CurrentTime + 1440;
        add_temp_kline(aconf);
      }
    }
  }
  sendto_one(source_p, ":%s NOTICE %s :%d users removed", me.name, parv[0],
             count);
#ifndef CONFIG_H_LEVEL_7
  check_dlines();
#endif
  check_klines();
}

